@@ -4,7 +4,7 @@
 */
 //
 //    BEDOPS
-//    Copyright (C) 2011-2017 Shane Neph, Scott Kuehn and Alex Reynolds
+//    Copyright (C) 2011-2018 Shane Neph, Scott Kuehn and Alex Reynolds
 //
 //    This program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -31,8 +31,8 @@ namespace Visitors {
   // Visitor that should be inherited when using sweep() algorithm.
   template <typename Ref, typename Map = Ref>
   struct Visitor {
-    typedef const Ref RefType;
-    typedef const Map MapType;
+    typedef Ref RefType;
+    typedef Map MapType;
 
     // interface for sweep()
     inline bool ManagesOwnMemory() const { return(false); }
