@@ -65,7 +65,7 @@ namespace Bed {
     allocate_iterator_starch_bed(Ext::FPWrap<ErrorType>& fp, Ext::PooledMemory<BedType, SZ>& p,
                                       const std::string& chr = "all") /* this ASSUMES fp is open and meaningful */
       : fp_(fp), _M_ok(fp_ && !std::feof(fp_)), _M_value(0),
-        is_starch_(_M_ok && (fp_ != stdin) && starch::Starch::isStarch(fp_)),
+        is_starch_(false),
         all_(0 == std::strcmp(chr.c_str(), "all")), archive_(NULL), pool_(&p) {
 
       chr_[0] = '\0';
@@ -86,7 +86,7 @@ namespace Bed {
           throw(ErrorType("Error: stat() failed on: " + fp.Name()));
         is_namedpipe = (S_ISFIFO(st.st_mode) != 0);
       }
-      is_starch_ = (is_starch_ && !is_namedpipe);
+      is_starch_ = !is_namedpipe && _M_ok && (fp_ != stdin) && starch::Starch::isStarch(fp_);
 
       if ( (fp_ == stdin || is_namedpipe) && !all_ ) { // BED, chrom-specific, using stdin
         // stream through until we find what we want
